CSV_NAME = 'bad_crc_to_PER_quantization.csv'
INTERFERENCE_ANALYSIS_FW_VER = "4.2.0"
CHANNELS_TO_ANALYZE = [(37, 2402), (38, 2426), (39, 2480)]
CNTRS_LISTEN_TIME_SEC = 30
MAX_UNSIGNED_32_BIT = 4294967295
NON_WLT_RX = 'non_wlt_rx'
WLT_RX = 'wlt_rx'
BAD_CRC = 'bad_crc'
CNTRS_KEYS = [NON_WLT_RX, WLT_RX, BAD_CRC]