RX_CHANNELS = [37, 38, 39]
RX_CHANNELS_SNIFFING_KIT = list(range(40))
SET_SNIFFER = '!set_logger_mode 2'
CANCEL_SNIFFER = '!set_logger_mode 0'
GET_LOGGER_COUNTERS = '!get_logger_counters'
GATEWAY_APP = '!gateway_app'
STOP_ADVERTISING = '!stop_advertising'
CANCEL = '!cancel'
RESET_GW = '!reset'
VERSION = '!version'
SEND_ALL_ADV_CHANNELS = 0
SERIAL_TIMEOUT = 0.1
SEP = "#"*100
LENGTH = '1E'
GAP_TYPE = '16'
DEFAULT_ADVA = 'FFFFFFFFFFFF'
DEFAULT_DUPLICATES = 3
GW_APP_VERSION_HEADER = 'WILIOT_GW_BLE_CHIP_SW_VER'
LOCATION = 'location'
DEFAULT_OUTPUT_POWER = 8
DEFAULT_DELAY = 20
BRIDGES = [i for i in range(3)]
MAX_NFPKT = 65535
MAX_RSSI = 255
ADVA_LENGTH = 12
GAP_LENGTH = 4
SERVICE_UUID_LENGTH = 4
GROUP_ID_LENGTH = 6
UPLINK_DUPLICATIONS = [i for i in range(3, 7)]
UPLINK_TIME_DELAYS = [20, 130, 255]
UNIFIED_DUPLICATIONS = [i for i in range(3, 7)]
UNIFIED_TIME_DELAYS = [20, 130, 255]
DEFAULT_LOOKOUT_TIME = 2
